/*

	MakeUEF by Thomas Harte 2000

	Distributed under the GPL - see 'Copying' for details

	Contains the code for reading from the sound card on various platforms

*/
#include "defines.h"
#include <stdlib.h>

#define BUF_SIZE 4096

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <mmsystem.h>
#include <conio.h>

HWAVEIN dev;
volatile WAVEHDR hdr1, hdr2, *chdr;
char *inbuf1, *inbuf2;
bool quit;

void C_sndcard::NewBlock(void)
{
	while(!(chdr->dwFlags&WHDR_DONE));
	waveInAddBuffer(dev, (WAVEHDR *)chdr, sizeof(WAVEHDR));

	data = chdr->lpData;
	pos = 0;
	chdr = chdr == &hdr1 ? &hdr2 : &hdr1;
}

bool C_sndcard::Open(char *name)
{
	WAVEFORMATEX devformat;
	WAVEINCAPS devcaps;
	MMRESULT res;
	int numdevs, seldev;
	int slevel = 0, nlevel;

	if((numdevs = waveInGetNumDevs()) < 1)
	{
		error = "No wave input devices installed";
		return false;
	}

	seldev = -1;
	while(numdevs--)
	{
		waveInGetDevCaps(numdevs, &devcaps, sizeof(WAVEINCAPS));

		nlevel = 0;
		if(devcaps.dwFormats & WAVE_FORMAT_2M08) //8bit mono, 22050Hz
			nlevel = 1;

		if(devcaps.dwFormats & WAVE_FORMAT_4M08) //8bit mono, 44100Hz
			nlevel = 2;

		if(nlevel > slevel)
		{
			slevel = nlevel;
			seldev = numdevs;
		}
	}

	if(seldev == -1)
	{
		error = "No suitable devices found (1)";
		return false;
	}

	freq = (slevel == 2) ? 44100 : 22050;

	devformat.wFormatTag = WAVE_FORMAT_PCM;
	devformat.nChannels = 1;
	devformat.nSamplesPerSec = 
	devformat.nAvgBytesPerSec = freq;
	devformat.nBlockAlign = 1;
	devformat.wBitsPerSample = 8;
	devformat.cbSize = 0;

	if((res = waveInOpen(&dev, WAVE_MAPPER, &devformat, 0, NULL, CALLBACK_NULL)) != MMSYSERR_NOERROR)
	{
		error = "No suitable wave devices found (2)";
		return false;
	}

	waveInStart(dev);

	inbuf1 = (char *)malloc(BUF_SIZE);
	inbuf2 = (char *)malloc(BUF_SIZE);
	quit = false;

	bufsize = BUF_SIZE;

	hdr1.lpData = inbuf1;
	hdr1.dwBufferLength = BUF_SIZE;
	hdr1.dwFlags = 0;
	hdr2.lpData = inbuf2;
	hdr2.dwBufferLength = BUF_SIZE;
	hdr2.dwFlags = 0;

	waveInPrepareHeader(dev, (WAVEHDR *)&hdr1, sizeof(WAVEHDR));
	waveInPrepareHeader(dev, (WAVEHDR *)&hdr2, sizeof(WAVEHDR));

	waveInAddBuffer(dev, (WAVEHDR *)&hdr1, sizeof(WAVEHDR));
	waveInAddBuffer(dev, (WAVEHDR *)&hdr2, sizeof(WAVEHDR));
	chdr = &hdr1;

	NewBlock();

	return true;
}

bool C_sndcard::Close(void)
{
	while(!(chdr->dwFlags&WHDR_DONE));
	chdr = chdr == &hdr1 ? &hdr2 : &hdr1;
	while(!(chdr->dwFlags&WHDR_DONE));

	waveInUnprepareHeader(dev, (WAVEHDR *)&hdr2, sizeof(WAVEHDR));
	waveInUnprepareHeader(dev, (WAVEHDR *)&hdr2, sizeof(WAVEHDR));

	waveInStop(dev);
	waveInClose(dev);

	free(inbuf1);
	free(inbuf2);
	return true;
}

bool C_sndcard::Finished(void)
{
	if(_kbhit())
	{
		switch(_getch())
		{
			default :
			break;

			case 0x1b :
			case 0x51 :
			case 0x71 :
				quit = true;
			break;
		}
	}

	return quit;
}

int C_sndcard::GetNextValue(void)
{
	int ret = -1; 

	ret = (unsigned char) data[pos];
	pos++;

	if(pos == bufsize)
	{
		NewBlock();
	}

	return ret;
}
